#include "includes.h"
#include "sftpconf.h"
#include "misc.h"
#include "log.h"
#include "xmalloc.h"
#include "pathnames.h"

/* Initializes the server options to their default values. */

void
initialize_sftp_options(SftpOptions *options)
{
	options->allow_ssh = 1;
	options->defaultdir = NULL;
	options->hide_num = 0;
	options->hide_list = NULL;
	options->show_num = 0;
	options->show_list = NULL;
	options->ro_num = 0;
	options->ro_list = NULL;
	options->rw_num = 0;
	options->rw_list = NULL;
}

void
read_sftp_config(SftpOptions *options, struct passwd *pw)
{
	int found = 0, isgrp, isusr, i;
	char line[1024], *cp, *arg, *value1;
	FILE *f = NULL;
	struct group *grp;

	if ((f = fopen(_PATH_SFTP_CONFIG_FILE, "r")) == NULL) {
		error("%s: %s", _PATH_SFTP_CONFIG_FILE, strerror(errno));
		goto out;
	}

	while (fgets(line, sizeof(line), f)) {
		cp = line;
		arg = strdelim(&cp);

		if (*arg == '\0')
			arg = strdelim(&cp);
		if (!arg || !*arg || *arg == '#')
			continue;

		if ((isusr = strcasecmp(arg, "user")) == 0 ||
		    (isgrp = strcasecmp(arg, "group")) == 0) {
			if (found)
				goto out;
			value1 = strdelim(&cp);

			if (!value1 || *value1 == '\0')
				continue;
			if (isusr == 0) {
				if (strcmp(value1, pw->pw_name) != 0)
					continue;
			} else if (isgrp == 0) {
				if ((grp = getgrnam(value1)) == NULL)
					continue;

				for (i = 0; grp->gr_mem[i] != NULL; i++) {
					if (strcmp(grp->gr_mem[i], pw->pw_name) == 0)
						break;
 				}
				if (grp->gr_mem[i] == NULL && grp->gr_gid != pw->pw_gid)
					continue;
			}

			found = 1;
			options->defaultdir = xstrdup(pw->pw_dir);
			options->allow_ssh = 0;
		} else if (strcasecmp(arg, "allowssh") == 0) {
			value1 = strdelim(&cp);
			if (!found || !value1 || *value1 == '\0')
				continue;
			else if (strcasecmp(value1, "yes") == 0)
				options->allow_ssh = 1;
			else if (strcasecmp(value1, "no") == 0)
				options->allow_ssh = 0;
		} else if (strcasecmp(arg, "defaultdir") == 0) {
			value1 = strdelim(&cp);
			if (!found || !value1 || *value1 == '\0')
				continue;
			if (options->defaultdir)
				xfree(options->defaultdir);
			options->defaultdir = xstrdup(value1);
		} else if (strcasecmp(arg, "hidelist") == 0) {
			value1 = strdelim(&cp);
			if (!found || !value1 || *value1 == '\0')
				continue;
			if (options->hide_num % 10 == 0) {
				options->hide_list = xrealloc(options->hide_list, sizeof(char*) * (options->hide_num + 10));
			}
			options->hide_num++;
			options->hide_list[options->hide_num-1] = xstrdup(value1);
		} else if (strcasecmp(arg, "showlist") == 0) {
			value1 = strdelim(&cp);
			if (!found || !value1 || *value1 == '\0')
				continue;
			if (options->show_num % 10 == 0) {
				options->show_list = xrealloc(options->show_list, sizeof(char*) * (options->show_num + 10));
			}
			options->show_num++;
			options->show_list[options->show_num-1] = xstrdup(value1);
		} else if (strcasecmp(arg, "rolist") == 0) {
			value1 = strdelim(&cp);
			if (!found || !value1 || *value1 == '\0')
				continue;
			if (options->ro_num % 10 == 0) {
				options->ro_list = xrealloc(options->ro_list, sizeof(char*) * (options->ro_num + 10));
			}
			options->ro_num++;
			options->ro_list[options->ro_num-1] = xstrdup(value1);
		} else if (strcasecmp(arg, "rwlist") == 0) {
			value1 = strdelim(&cp);
			if (!found || !value1 || *value1 == '\0')
				continue;
			if (options->rw_num % 10 == 0) {
				options->rw_list = xrealloc(options->rw_list, sizeof(char*) * (options->rw_num + 10));
			}
			options->rw_num++;
			options->rw_list[options->rw_num-1] = xstrdup(value1);
		}

	}
 out:
	if (f)
		fclose(f);
}
